# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2002-2003 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2003 Itaapy <contact@itaapy.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


"""
The Itaapy Tools.
"""

__revision__ = "$Id: setup.py,v 1.15 2003/10/22 08:59:01 jdavid Exp $"


# Import Python modules
from distutils.command import build_py
from distutils.core import setup
from glob import glob
from types import StringType
import os
import string



class build_py(build_py.build_py):
    """
    Provides a version of the Distutils 'build_py' command that knows about
    DTML and text files.

    Copied from quixote.qx_distutils.py

    This bites -- way too much code had to be copied from
    distutils/command/build.py just to add an extra file extension!
    """

    def find_package_modules(self, package, package_dir):
        self.check_package(package, package_dir)
        module_files = (glob(os.path.join(package_dir, "*.py")) +
                        glob(os.path.join(package_dir, "*.dtml")) +
                        glob(os.path.join(package_dir, "*.txt")))
        modules = []
        setup_script = os.path.abspath(self.distribution.script_name)

        for f in module_files:
            abs_f = os.path.abspath(f)
            if abs_f != setup_script:
                module = os.path.splitext(os.path.basename(f))[0]
                modules.append((package, module, f))
            else:
                self.debug_print("excluding %s" % setup_script)
        return modules


    def build_module(self, module, module_file, package):
        if type(package) is StringType:
            package = string.split(package, '.')
        elif type(package) not in (ListType, TupleType):
            raise TypeError, \
                  "'package' must be a string (dot-separated), list, or tuple"

        # Now put the module source file into the "build" area -- this is
        # easy, we just copy it somewhere under self.build_lib (the build
        # directory for Python source).
        outfile = self.get_module_outfile(self.build_lib, package, module)
        if module_file.endswith(".dtml"): # XXX hack for DTML
            outfile = outfile[0:outfile.rfind('.')] + ".dtml"
        elif module_file.endswith(".txt"): # XXX hack for TXT
            outfile = outfile[0:outfile.rfind('.')] + ".txt"
        dir = os.path.dirname(outfile)
        self.mkpath(dir)
        return self.copy_file(module_file, outfile, preserve_mode=0)



# XXX itools.zope.zmi shouldn't be a Python package, but this is the
# quickest workaround I found to install this directory.
setup(name = "itools",
      version = "0.2.3",
      author = "J. David Ibez",
      author_email = "jdavid@itaapy.com",
      license = "GNU Lesser General Public License",
      url = "http://sf.net/projects/lleu",
      description="Miscellaneous tools: Flux (workflow), the Simple Template Language, etc..",
      package_dir = {'itools': ''},
      packages = ['itools', 'itools.catalog', 'itools.i18n', 'itools.workflow',
                  'itools.xml', 'itools.zope', 'itools.zope.zmi'],
##      data_files = [('zope/zmi', ['zope/zmi/localroles.dtml'])],
      cmdclass = {'build_py': build_py})
